/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/******************************************************************************
 * This file contains the main program of the command listener; it initializes
 * the user requested configuration of the eshell and the communication 
 * channels and handles the communication
 *****************************************************************************/

#include <stdio.h>
#include "listener.h"
#include <strings.h>
#include <exFkm.h>
#include "cmds.h"
#include "appl.h"
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <errno.h>

extern errno;

Listener  *listener;
Display *display;

fd_set readfs[2];	       
static Boolean GOON = True;
static char msg[128];

main(argc,argv)
     int argc;
     char *argv[];
{ 
  int i, nfds;
  void insertMenuString();
  void submitCmd();
  int maxfd = getdtablesize();
  char line[513];
  register rc;
  register char *ptr;
  EditorDescr lview, lcommand;
  int eshErrorHandler();
  int eshIOErrorHandler();
  void cleanUpEsh();
  int displayTime();
  Widget top;
  int dpyno;
  char *geometry = ESH_GEOMETRY;
  Boolean appl_init = True;

  BufInfo *bufinfoptr();

  i = 1;
  while (i < argc) {
    if (strcmp(argv[i], "-geometry") == 0)
      geometry = argv[++i];
    else if (strcmp(argv[i], "-n") == 0) 
      appl_init = False;
    
    i++;
  }

  top = XtInitialize(NULL, "ESH", (XrmOptionDescRec *) NULL, 0, &argc, argv);

  display = XtDisplay(top);
  dpyno = XConnectionNumber(display);
  XSetErrorHandler(eshErrorHandler);
/*
  signal(SIGALRM, displayTime); 
*/

  signal(SIGQUIT, cleanUpEsh);
  XSetErrorHandler(eshErrorHandler);
  XSetIOErrorHandler(eshIOErrorHandler);

  exInitLayout(display);
  exInitMenus(display);
  exInitFKM(display);
  exInitMacro(top);
  initCLdescriptions(&lview, &lcommand);
  initMHfunctions();

  listener = createListener(top, &lview, &lcommand, geometry);
  
  initCmdTables(listener->view, viewCmdTable, viewEscCmds, 
					ctrlXCmds,(cmdDescr *) 0);
  initCmdTables(listener->command, editCmdTable, editEscCmds, ctrlXCmds, clCmds);

  initApplications(listener, appl_init);

  show_eshell(listener);

  eshSetSizeHints(listener);

  listener->cmdIndex = 1;
  adjustReadMask(listener);

  doWritePrompt(listener);
  displayDate(listener);

  while (GOON) {
    XFlush(display);
    processTerminatedChildren(listener);

    bcopy((char *) &readfs[0], (char *) &readfs[1], sizeof(readfs[0])); 
    
    nfds = select(maxfd, &readfs[1], 
		  (fd_set *) 0, (fd_set *) 0, (struct timeval *) 0); 

    if (nfds < 0) {
      if (errno == EINTR)			/* caused by alarm interupt */
	continue;
      else {
	sprintf(msg, "select error: %d\n", errno);
	fatal("ESH", msg, NULL);
      }
     }

    if (FD_ISSET(dpyno, &readfs[1])) 		/* process events */
      processEvents();
      
    for (i = 0; i < ESH_MAXAPPL; i++) {		/* process components */
      if (listener->application[i].fd <= 0 || 
	  	listener->application[i].pid <= 0)
	continue;

      if (FD_ISSET(listener->application[i].fd, &readfs[1])) {
	switch (i) {
	case SHELL:				/* process shell output */
	  if ((rc = read(listener->application[SHELL].fd, line, 512)) > 0) {
	    *(line + rc) = EOS;
	    insertShellOutput(line, listener->view);
	  } else if (rc < 0) {
	    sprintf(msg, "Reading %s: ", 
		    		listener->application[SHELL].procName);
	    perror(msg);
	    eshQuit((RequestParm *) NULL);
	  }
	  break;
	case IFEDIT:				/* process IFE-messages */
	  if (processIFEmsg(listener->application[IFEDIT].fd) < 0) {
	    sprintf(msg, "Error reading %s: ", 
		    		listener->application[IFEDIT].procName);
	    eshMsg(msg);
	  }
	  break;
	case HTRACE:				/* insert command-line */
	  if ((rc = read(listener->application[i].fd, line, 512)) > 0) {
	    *(line + rc) = EOS;
	    editorFillBuffer(listener->command, line, True);
	    putCursor(listener->command);
	  } else if (rc < 0) {
	    sprintf(msg, "Error reading %s: killed !", 
		    listener->application[i].procName);
	    eshMsg(msg);
	    killApplication(HTRACE, False);
	  }
	  break;
	default:  			/* process remaining components */
	  if ((rc = read(listener->application[i].fd, line, 512)) > 0) {
	    *(line + rc) = EOS;
	    ptr = line;
	    editorInsert(listener->command, ptr);
	    putCursor(listener->command);
	  } else if (rc < 0) {
	    sprintf(msg, "Error reading %s: killed !", 
		    listener->application[i].procName);
	    eshMsg(msg);
	    killApplication(i, False);
	  }
	}
      }
    }
  }

  XFlush(display);
  processEvents();
  exReset(display);
  XCloseDisplay(display);  
  return(0);
}  

/* processTerminatedChildren: *************************************************
 * look if any children have terminated and adjust the listener data structues
 * appropriately
 *****************************************************************************/

processTerminatedChildren(l)
     Listener *l;
{
  int pid;
  register i;
  union wait status;

  while ((pid = wait3(&status, WNOHANG, (struct rusage *) NULL)) > 0)
    for (i = 0; i < ESH_MAXAPPL; i++) 
      if (pid == l->application[i].pid) {
	if (l->application[i].flags & ESH_CONNECT)
	  (void) close(l->application[i].fd);
	l->application[i].fd = -1;
	l->application[i].pid = -1;
	l->application[i].flags = ESH_OCCUPIED;
	adjustReadMask(l);
	sprintf(msg,"-- %s -- terminated !", l->application[i].procName);
	eshMsg(msg);
	break;
      }
}

/* _quitEsh: ******************************************************************
 * set the boolean variable `GOON' that controls when to leave the esh; if this
 * variable is False, the main loop is terminated (see function main().
 *****************************************************************************/

_quitEsh()
{
  GOON = False;
}

/* processEvents: *************************************************************
 * This function is called when any events are to be processed; it 
 * subsequently reads events from the event-queue and processes them. A 
 * distinction is made between `normal' X events and `property events', 
 * that are generated on behalf of the menu-handling system, the function 
 * system, the macro system and the command listener. The latter events are 
 * used to transfer information between the clients and the listener.
 ****************************************************************************/

processEvents()
{	/* process all pending events */
  XEvent event;

  while (XtPending()) {				/* process each event */
    XtNextEvent(&event);
    XtDispatchEvent(&event);
  }
}

/* processIFEmsg: *************************************************************
 * process the message sent by the interface editor 
 *****************************************************************************/

processIFEmsg(fd)
     int fd;
{
  register i;
  static char CLOSE_MSG[] = {ctrl('x'), ctrl('x'), EOS};
  static char QUIT_MSG[]   = {ctrl('x'), ctrl('c'), EOS};
  static int QUIT_SIZE = 0, CLOSE_SIZE = 0;
  int pid, opcode, info;

  if (! QUIT_SIZE)
    QUIT_SIZE = strlen(QUIT_MSG);
  if (! CLOSE_SIZE)
    CLOSE_SIZE = strlen(CLOSE_MSG);


  if (exReceiveMsg(fd, &pid, &opcode, &info) < 0)
    return(-1);

  switch (opcode) {
  case IFE_QUIT:			/* send a quit request */
    for (i = 0; i < ESH_MAXAPPL; i++) {
      if (i == SHELL || i == IFEDIT)	/* makes no sense to the shell */
	continue;
      if (listener->application[i].pid == pid 
	  && listener->application[i].flags & ESH_OCCUPIED) {
	killApplication(i, False);
	
	sprintf(msg, "Application -- %s -- killed !", 
				listener->application[i].procName);
	eshMsg(msg);
	break;
      }
    }
    eshMsg("Illegal id in message from interface editor !");
    break;
  default:
    eshMsg("Illegal opcode in message from interface editor !");
    break;
  }
  return(0);
}

/* adjustReadMask: ***********************************************************
 * adjust the mask used to listen to the file-descriptors of all connected 
 * applications.
 ****************************************************************************/

adjustReadMask(l)
     Listener *l;
{
  register i;

  FD_ZERO(&readfs[0]);
  FD_SET(l->dpyno, &readfs[0]);

  for (i = 0; i < ESH_MAXAPPL; i++) 
    if (l->application[i].flags & ESH_STARTED && 
	l->application[i].flags & ESH_CONNECT)
      FD_SET(l->application[i].fd, &readfs[0]);
}

/* displayDate: ***************************************************************
 * It displays the current date in the prompt window.
 *****************************************************************************/
/*ARGSUSED*/
displayDate(l)
     Listener *l;
{
  time_t esh_time;
  char *time_str, *ctime();
  char month[5], dow[5];
  int day, year, dummy;

  if (time(&esh_time) < 0) {
    eshMsg("Cannot retrieve time of day !");
  } else {
    time_str = strCopy(ctime(&esh_time));
    sscanf(time_str, "%s %s %d %d:%d:%d %d\n", dow, month, &day, 
	   &dummy, &dummy, &dummy, &year);

    sprintf(msg, "%s %s %d", dow, month, day);

    XDrawImageString(l->display, XtWindow(l->prompt), 
		     l->gc, 1, 4 * l->font->max_bounds.ascent, 
		     msg, strlen(msg));

    sprintf(msg, "%d", year);

    XDrawImageString(l->display, XtWindow(l->prompt), 
		     l->gc, 15, 5 * l->font->max_bounds.ascent, 
		     msg, strlen(msg));
  }
  free(time_str);
}

/*ARGSUSED*/
eshErrorHandler(d,e)
     Display *d;
     XErrorEvent *e;
{
  kill(getpid(), SIGQUIT);
}

/*ARGSUSED*/
eshIOErrorHandler(d)
     Display *d;
{
  kill(getpid(), SIGQUIT);
}
/*ARGSUSED*/
void cleanUpEsh(sig, code, scp, addr)
     int sig;
     int code;
     struct sigcontext *scp;
     char *addr;
{
  register i;

  for (i = 0; i < ESH_MAXAPPL; i++) 			/* kill all kids */
    killApplication(i, (i == SHELL) ? True : False);	

  kill(getpid(), SIGQUIT);				/* force a core dump */
}
  
