/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <exLayout.h>
#include <sys/types.h>
#include "exEdit.h"
#include "exEntry.h"
#include "procs"

static char SccsID[] = "@(#)help.c	1.2\t11/4/92";            /* sccs keyword */

static cmdDescr cmdHlp[] = {			/* messages for help */
  {left,            "  Move cursor left one character."},
  {right, 	    "  Move cursor right one character."},
  {beglin, 	    "  Move cursor to the beginning of the current line."},
  {endlin, 	    "  Move cursor to the end of the current line."},
  {upcol, 	    "  Move cursor up in column."},
  {dwncol, 	    "  Move cursor down in column."},
  {wordl, 	    "  Move cursor back to the beginning of the previous word."},
  {wordr, 	    "  Move cursor forward to the end of the next word."},
  {topscr, 	    "  Move cursor to the first line of the screen."},
  {botscr, 	    "  Move cursor to the last line of the screen."},
  {midscr, 	    "  Move cursor to a line near the center of the screen."},
  {goLine, 	    "  Move cursor to a specific line. You are prompted to enter a line number."},

  {uplin, 	    "  Move text up one line, retain cursor position within the screen."},
  {dwnlin, 	    "  Move text down one line, retain cursor position within the screen."},
  {upScroll, 	    "  Display next screen of the text (scroll up)."},
  {dwnScroll, 	    "  Display previous screen of the text (scroll down)."},
  {topBuffer, 	    "  Move cursor to the first line of the buffer."},
  {botBuffer, 	    "  Move cursor to the last line of the buffer."},
  {lineToTop, 	    "  Scroll text until the current line is at the top of the screen."},
  {dwnpage, 	    "  Display previous page of the text."},
  {uppage, 	    "  Display next page of the text."},
  {uphpage, 	    "  Scroll text up half a page."},
  {dwnhpage, 	    "  Scroll text down half a page."},

  {newline,         "  Insert a newline character to break the current line."},
  {openln, 	    "  Insert an empty line above the current one."},
  {openlnd,	    "  Insert an empty line below the current one."},
  {delprev, 	    "  Delete previous character."},
  {delnxt, 	    "  Delete next character."},
  {bdelword, 	    "  Delete previous word."},
  {fdelword, 	    "  Delete next word."},
  {krest, 	    "  Delete from the current cursor position to the end of the line."},

  {dot,  	    "  Set the dot to the current position."},
  {exMark,          "  Set the mark to the current position and mark the region as selected."},
  {exchangeDotMark, "  Exchange dot and mark thereby moving the cursor to the position as indicated by the mark."},
  {copy, 	    "  Copy the current region into the kill buffer."},
  {cut, 	    "  Cut the current region into the kill buffer."},
  {yank, 	    "  Paste (yank) the contents of the kill buffer to the current position."},
  {resetRegion,	    "  Reset region description."},

  {insertFile, 	    "  Insert a file in the current buffer at the cursor position. You are asked to enter the name of the file."}, 
  {findFile, 	    "  Edit another file. You are prompted to enter the name of the file."},
  {saveFile, 	    "  Save the contents of the current buffer to the corresponding file. If no current file is specified, you are asked to enter the name of the file."},
  {writeFile, 	    "  Store the contents of the current buffer in a file. You are prompted to enter the name of the file."},

  {memory, 	    "  Display statistics: cursor position relative to the screen and the buffer, the maximal number of lines of the current buffer and the yidentifier of this buffer."},
  {whatLine,	    "  Display the position (line and column) of the cursor relative to the screen and the buffer."},
  {help, 	    "  Print a list of currently available functions with a short explanation: purpose of the function and how it is invoked."},

  {fsearch, 	    "  Search string forward. You are prompted to enter the search string."},
  {bwsearch, 	    "  Search string backward. You are prompted to enter the search string."},
  {ffsearch, 	    "  Continue search forward with the previous search string."},
  {bbsearch, 	    "  Continue search backward with the previous search string."},

  {editExit, 	    "  Exit the editor. You are asked if any changes need to be saved."},
  {editAbort, 	    "  Abort the editor. You are NOT asked if any changes need to be saved."},
  {newscr, 	    "  Redraw the screen."},
  {clearBuffer,	    "  Clear the current edit buffer. You are asked if any changes need to be saved."},
  {killBuffer, 	    "  Clear the current edit buffer. You are NOT asked if any changes need to be saved."},

  {noop, 		"  no function provided"},
  {(int (*)()) 0, (char *) 0}
};

/* make_help: ****************************************************************
 * generate the help window that contains a short command index 
 ****************************************************************************/

make_help(bf, addCmdTable)
     BufInfo *bf;
     cmdDescr *addCmdTable;
{
  Editor makeStxt();
 
  bf->hlp = makeStxt(bf->w, "Help Information", 60, 20, 200);
  
  clearBuffer(bf->hlp);

  editorAppend(bf->hlp, "Help buffer: valid commands and descriptions!"); 
  editorAppend(bf->hlp, "\n\n\nCommands:\n\n");
  
  writeCommandTable(bf->hlp, addCmdTable, bf->cmdTable, "");
  writeCommandTable(bf->hlp, addCmdTable, bf->escTable, "ESC");
  writeCommandTable(bf->hlp, addCmdTable, bf->cxTable, "^x" );

  editorAppend(bf->hlp, "\n\n\nCurrent mouse bindings:\n\n");
  editorAppend(bf->hlp, "Btn1 	    	mouse pos\n");
  editorAppend(bf->hlp, "Btn2       	mouse select\n");
  editorAppend(bf->hlp, "Btn3       	mouse deselect\n");
  editorAppend(bf->hlp, "Shift Btn1 	mouse copy\n");
  editorAppend(bf->hlp, "Shift Btn2 	mouse cut\n");
  editorAppend(bf->hlp, "Shift Btn3 	mouse paste\n");

  setViewMode(bf->hlp);
}

/* writeCommandTable: ********************************************************
 * generate a readable form of the specified command table: print the 
 * key-combination of each command together with a short command description
 ****************************************************************************/

writeCommandTable(hlp, cmds, table, prefix)
     Editor hlp;
     register cmdDescr *cmds;
     editCmd *table;
     char *prefix;
{
  register editCmd *cmdPtr;
  register cmdDescr *descrPtr;
  char pref[5];
  char cmdChar[4];
  char cmd[11];
  register found = 0;
  static char NEWLINE[2] = {'\n',EOS};
  
  if (prefix && *prefix) 
    sprintf(pref, "%s-", prefix);
  else
    pref[0] = EOS;
  
  for (cmdPtr = table; cmdPtr->func != (int (*)()) 0; cmdPtr++) {
    found = 0;
    printable(cmdPtr->key, cmdChar);
    sprintf(cmd, "%s%s\t", pref, cmdChar);
    editorAppend(hlp, cmd);
    
    for (descrPtr = cmdHlp; descrPtr->func != (int (*)()) 0; descrPtr++) {
      if (descrPtr->func == cmdPtr->func) {
	editorAppend(hlp, descrPtr->txt);
	found = 1;
	break;
      }
    }
    
    if (! found && cmds) {		/* search user supplied table */
      for (descrPtr = cmds; descrPtr->func != (int (*)()) 0; descrPtr++) {
	if (descrPtr->func == cmdPtr->func) {
	  editorAppend(hlp, descrPtr->txt);
	  found = 1;
	  break;
	}
      }
    }    
    
    if (descrPtr->func == (int (*)()) 0) 
      editorAppend(hlp, "illegal function (internal error)");
    
    editorAppend(hlp, NEWLINE);
  }
}    

/* printable: ***************************************************************
 * generate a printable representation of a character
 ****************************************************************************/

printable(chr, str)
     char chr;
     char *str;
{	
  
  if (chr < ' ' && chr > EOS)             /* control character (001 - 037) */ 
    sprintf(str, "^%c", chr + '`');
  else if (chr == 127)                    /* delete character (0177) */
    sprintf(str, "%s", "DEL");
  else                                    /* visible graphic character
					     (040 - 0176) */
    sprintf(str, "%c", chr);
}
