/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>

#include <stdio.h>
#include <sys/types.h>
#include <exBasics.h>
#include "exEdit.h"
#include <exLayout.h>

#include <X11/Shell.h>
#include <Xm/Form.h>

static char SccsID[] = "@(#)stxt.c	1.2\t11/10/92";            /* sccs keyword */

BufInfo *bufinfoptr();
int stxtClose();

static Cursor helpCursor;
static void stxtTitle();
static void initStxt();
static stxtPrint();

static EditorDescr stxtDescr = {
  (int (*)()) 0,
  (char *) 0,
  "stxt-editor",
  (char **) 0,
  (Window) 0,
  0,					/* number of lines in buffer */
  0,					/* lines in window */
  0,					/* cols  in window */
  0, 					/* width of window */
  0,					/* height of window */
  ED_SCROLLBAR | ED_LINEBREAK	   	/* flags */
};

static editCmd stxtCmdTable [] = {
  {ctrl('f'), right},		/* cursor right */
  {ctrl('b'), left},		/* cursor left  */
  {ctrl('n'), dwncol},		/* down in column */
  {ctrl('p'), upcol},		/* up in column */
  {ctrl('a'), beglin},		/* beginning of line */
  {ctrl('e'), endlin},		/* end of line */
  {ctrl('v'), uppage},		/* forward page */
  {ctrl('l'), newscr},		/* redraw screen */
  {ctrl('s'), fsearch},		/* search forward */
  {ctrl('r'), bwsearch},		/* search backward */
  {EOS,     (int (*)()) 0},	/* end of list */
};		

static editCmd ctrlxTable [] = {		       
  {ctrl('c'), stxtClose},  	/* abort editor */
  {'c', stxtClose},		/* leave editor */
  {'t', lineToTop},		/* current line to top of screen */
  {'p', stxtPrint},		/* print current buffer */
  {EOS, (int (*)()) 0},		
};

static editCmd escTable [] = {
  {'<', topBuffer},		/* goto top of file */
  {'>', botBuffer},		/* goto bottom of file */
  {' ', goLine},		/* goto specified line */
  {'=', memory},		/* display statistics */
  {'b', wordl},			/* move to left word */
  {'f', wordr},			/* move to right word */
  {'v', dwnpage},		/* display previous page */
  {'n', upScroll},		/* scroll up */
  {'p', dwnScroll},		/* scroll down */
  {'l', botscr},		/* cursor to last line of display */
  {'m', midscr},		/* cursor near middle of display */
  {'h', topscr},		/* cursor to top of display */
  {'s', ffsearch},		/* continue search forward */
  {'r', bbsearch},		/* continue search backword */
  {EOS, (int (*)()) 0},		
};

/* makeStxt: *****************************************************************
 * generate an instance of an editor for displaying static text 
 ****************************************************************************/

Editor makeStxt(parent, name, cols, lines, bufferLines)
     Widget parent;
     char *name;
     int cols, lines, bufferLines;
{
  Arg args[10];
  Widget w;
  Editor editor;
  register cnt = 0;

  initStxt(XtDisplay(parent));

  stxtDescr.lines = lines;
  stxtDescr.cols = cols;
  stxtDescr.bufferLines = bufferLines;
  
  XtSetArg(args[cnt], XmNborderWidth, 2); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  w = XtCreatePopupShell(name, transientShellWidgetClass, parent, args, cnt);
  
  editor = makeEditor(w, "stxt", &stxtDescr, 0, 0);
  initCmdTables(editor, stxtCmdTable, escTable, ctrlxTable, (cmdDescr *) 0);
  exEditSetCursor(editor, helpCursor);
  return(editor);
}

/* stxtShow: *****************************************************************
 * display the editor containing the help text 
 ****************************************************************************/

stxtShow(editor, x, y)
     Editor editor;
     int x, y;
{
  register BufInfo *bf = bufinfoptr(editor);

  XtMoveWidget(XtParent(bf->w), x, y);
  XtPopup(XtParent(bf->w), XtGrabNone);
}

/* stxtClose: ****************************************************************
 * close the help editor (unmap its window). As a last action a priovide 
 * function is called. E.g. if the stxt editor has been called as a help 
 * editor for an entry, this function resets the focus to the entry window.
 ****************************************************************************/

stxtClose(editor)
     Editor editor;
{
  register BufInfo *bf = bufinfoptr(editor);

  XtPopdown(XtParent(bf->w));
}

/* stxtPrint: ****************************************************************
 * print the contents of the help editor using the PRINT environment variable
 * the value of PRINT should be a valid print-command
 ****************************************************************************/

static stxtPrint(editor)
     Editor editor;
{
  char *print, *str;
  char *getenv();
  register FILE *fd;
  char fname[100], cmd[100];
  register BufInfo *bf = bufinfoptr(editor);

  if ((print = getenv("PRINT")) == (char *) 0) {
    exAlert(bf->w, -1, -1, 
	    "Cannot print: no \"PRINT\"-variable set in environment!");
    return;
  }

  sprintf(fname, "/tmp/stTXT.%d", getpid());
  if ((fd = fopen(fname, "w")) == (FILE *) 0) 
    return;

  if (! (str = getBufferContents(editor))) {
    exAlert(bf->w, -1, -1, 
	    "Cannot print: cannot get contents of text buffer !");
    return;
  }

  while (*str)
    fputc(*str++, fd);

  fclose(fd);

  sprintf(cmd, "%s %s &", print, fname);
  system(cmd);
}

/* initStxt: *****************************************************************
 * initialise the association table that associates an editor with its 
 * help editor
 ****************************************************************************/

static void initStxt(d)
     Display *d;
{
  static Boolean initialised = False;

  if (initialised)
    return;

  helpCursor = XCreateFontCursor(d, XC_question_arrow);

  initialised = True;
}

