/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include "exMenuP.h"
#include <exErrors.h>
#include "m_int.h"

/*****************************************************************************
 * this file contains some generally useful functions for the menu system
 *****************************************************************************/

static char msg[200];

/* countRequestsInPattern: ***************************************************
 * count '$'(stands for request) in return pattern ('$$' is not counted)
 ****************************************************************************/

countRequestsInPattern(pattern)
char *pattern;
{
  int cnt = 0;
  
  if (! pattern) 
    return(cnt);

  while (*pattern) {
    if (*pattern++ == '$') {
      if (*pattern != '$')
	cnt++;
      if (*pattern)
	pattern++;
    }
  }

  return(cnt);
}

/* legalType: ***************************************************************
 *
 ****************************************************************************/

Boolean legalType(type,value)
     char *type;
     int value;
{
  if (strcmp(type, "MDS_MENU_TYPE") == 0) 
    return(((MDS_MENU_TYPE) value == MDS_M_NULL   ||
	    (MDS_MENU_TYPE) value == MDS_M_POPUP  || 
	    (MDS_MENU_TYPE) value == MDS_M_PULLD  ||
	    (MDS_MENU_TYPE) value == MDS_L_POPUP  || 
	    (MDS_MENU_TYPE) value == MDS_L_PULLD) ? True : False);

  if (strcmp(type, "MDS_CNT_TYPE") == 0)
    return(((MDS_CNT_TYPE) value == MDS_CNT_NULL ||
	    (MDS_CNT_TYPE) value == MDS_CNT_EQ   ||
	    (MDS_CNT_TYPE) value == MDS_CNT_LT   ||
	    (MDS_CNT_TYPE) value == MDS_CNT_LE   ||
	    (MDS_CNT_TYPE) value == MDS_CNT_GT   ||
	    (MDS_CNT_TYPE) value == MDS_CNT_GE) ? True : False);

  if (strcmp(type,"MDS_POS_TYPE") == 0)
    return(((MDS_POS_TYPE) value == MDS_POS_NULL  ||
	    (MDS_POS_TYPE) value == MDS_POS_ABS   ||
	    (MDS_POS_TYPE) value == MDS_POS_CREL) ? True : False);

  if (strcmp(type,"MDS_REPR_TYPE") == 0)
    return(((MDS_REPR_TYPE) value == MDS_RTYP_NULL   ||
	    (MDS_REPR_TYPE) value == MDS_RTYP_STRING) ? True : False);

  setError(ERR_TYPE);
  return(False);
}

/* countCondition ***************************************************************
 * look if the number of selected items fulfills the restrictions
 * imposed by the selection count and the selection count type 
 *******************************************************************************/
  
countCondition(cond, first, second)
MDS_CNT_TYPE cond;
int first, second;
{
  switch (cond) {
  case MDS_CNT_EQ: 
    return(first == second);
  case MDS_CNT_LT:
    return(first < second);
  case MDS_CNT_LE:
    return(first <= second);
  case MDS_CNT_GT:
    return(first > second);
  case MDS_CNT_GE:
    return(first >= second);
  default:
    return(0);
  }
}

/* announceIllegalMenuCount **************************************************
 * display an appropriate error message
 ****************************************************************************/
  
void announceIllegalMenuCount(m, csr)
     exMenu *m;
     Cursor csr;
{
  sprintf(msg,"You can not leave the menu \nunless you have selected %s%d items;\ncurrently you have selected %d items.",
	  countType[(int)m->selType - (int)MDS_CNT_NULL - 1][0], 
	  m->selCnt, m->selItemCnt);

  exAlert(m->pane, -1, -1, msg);

}

/* announceIllegalTokenCount *************************************************
 * display an appropriate error message
 ****************************************************************************/

void announceIllegalTokenCount(m, parent, type, cnt, actCnt)
     exMenu *m;
     Widget parent;
     MDS_CNT_TYPE type;
     int cnt;
     int actCnt;
{
  sprintf(msg,"You can leave the input-mode only\nif you have provided %s%d tokens;\ncurrently you have input %d tokens.",	  
	  countType[(int)type - (int)MDS_CNT_NULL - 1][0], cnt, actCnt);
  
  exAlert(parent, -1, -1, msg);
}

/* countTokens ***************************************************************
 * count the tokens in a string (separated by blanks) that is typically 
 * returned by a request function in the menu system.
 ****************************************************************************/

int countTokens(str)
char *str;
{
  register char c;
  int cnt = 0;
  int blank = 0;
  
  while (*str == ' ')
    str++;

  if (*str == '\0')
    return(0);

  while ((c = *str++)) {
    if (c == ' ' && ! blank) {
      cnt++;
      blank = 1;
    } else if (c != ' ')
      blank = 0;
  }
  
  return(++cnt);
}
    
/* insertReturnValues: ******************************************************
 * insert the user supplied string into the item's return pattern; the $'s
 * in the return pattern are substituted by the corresponding return-value;
 ***************************************************************************/

insertReturnValues(iptr, pattern, buffer, returnbuffer)
     menuEntry *iptr;
     char *pattern;
     char *buffer[];
     char **returnbuffer;			/* returned parameter */
{

  register char *ptr, *bptr;
  int i, j, len;
  char c;

  if (! iptr->i_return || !pattern) {
    *returnbuffer = NULL;
    return;
  }

  len = strlen(iptr->i_return);

  for (i = 0; i < iptr->reqCnt; i++)
    if (buffer[i])
      len += strlen(buffer[i]);

  ptr = pattern;
  *returnbuffer = (char *) malloc(len + 1);

  bptr = *returnbuffer;

  for (i = 0; i < iptr->reqCnt; i++) {
    while (bptr < *returnbuffer + len) {
      if (ptr >= pattern + strlen(pattern))
	setError(ERR_MAX);
      if (*ptr == '$') {			/* expand '$' */
	c = *(ptr + 1);
	if (c == '$') {				/* insert a '$', dont expand */
	  *bptr++ = *ptr++;
	  ptr++;
	} else {				/* insert request value */
	  if (buffer[i]) {
	    for (j = 0; buffer[i][j]; j++)	/* expand '$' */
	      *bptr++ = buffer[i][j];
	  }
	  ptr++;
	  break;				/* next request value */
	}
      } else 					/* not a '$' */
	*bptr++ = *ptr++;			/* copy from pattern */
    }
  }
    
  /* now copy the remaining chars from the pattern */
  while (*ptr && bptr < *returnbuffer + len)
    *bptr++ = *ptr++;

  *bptr = EOS;
}

